<?php
/**
 * Plugin Name: MegaURL
 * Author: Nguyen Trong Vinh <ntvinh.it@gmail.com>
 * Author URL: http://ntvinh.ddns.net/
 */

/**
 * Class MegaURL
 * Singleton
 */
final class MegaURL {
	private static $instance = null;

	public static function instance() {
		if ( static::$instance === null ) {
			static::$instance = new MegaURL();
		}

		return static::$instance;
	}

	private function __construct() {
		// Add Menu
		if(isset($_REQUEST['action']) && is_callable($this->{$_REQUEST['action']})) {
			$this->{$_REQUEST['action']}();
		} else {
			$this->addMenu();
			$this->header();
		}
	}

	private function __clone() {
	}

	/**
	 * Alias of instance method
	 */
	public static function bootstrap() {
		static::instance();
	}

	protected function addMenu() {
		add_action( 'admin_menu', function () {
			add_submenu_page( 'options-general.php',
				'MegaURL',
				'MegaURL Settings',
				'administrator',
				'megaurl',
				[$this, 'adminTemplate']
			);
		} );
	}

	public function adminTemplate() {
		require __DIR__ . '/template.php';
	}

	public function saveData() {
		$data = $_REQUEST;
		unset($data['action'], $data['page']);
		update_option('megaurl', json_encode($data), true);
	}

	public function header() {
		add_action('wp_head', function() {
			?>
			<script type="text/javascript" id="megaurl-script">
(function(){
    function hashLink(link) {
        return 'https://megaurl.in/full/?api=' + token + '&url=' + btoa(link) + '&type=1';
    }
    var data = <?= get_option('megaurl'); ?>,
        token = data.token || '';
    if (!token) return null;
    var domains = (data.domains || '').split(/\r?\n/).map(function(domain) {return domain.trim()}),
        patterns = (data.patterns || '').split(/\r?\n/).map(function(pattern) {
            pattern = pattern.trim().split(' ');
            if (!pattern[0]) return null;
            return new RegExp(pattern[0], (pattern[1] || ''));
        }).filter(function(pattern) {return pattern});

    domains.push('megaurl.in');
    domains.push('www.megaurl.in');

    setInterval(function() {
        var aTags = document.querySelectorAll('a:not(.hashed)');
        aTags.forEach(function(el) {
            var href = el.getAttribute('href') || '';
            if (!href || href[0] === '#' || href[0] === '!' || href.substr(0,11) === 'javascript:') return false;
            var hashed = false;
            if (domains.indexOf(el.hostname) === -1) {
                el.href = hashLink(el.href);
                hashed = true;
                return null;
            }
            
            patterns.forEach(function(pattern) {
                if(pattern.test(el.href)) {
                    el.href = hashLink(el.href);
                    hashed = true;
                }
            });

            if (hashed) el.classList.add('hashed');
        });
    }, 200);
})();
			</script>
			<?php
		});
	}
}

MegaURL::bootstrap();